/*
 * Copyright 2015, Yahoo Inc.
 * Copyrights licensed under the New BSD License.
 * See the accompanying LICENSE file for terms.
 */

'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _types = require('../types');

var _utils = require('../utils');

var SECOND = 1000;
var MINUTE = 1000 * 60;
var HOUR = 1000 * 60 * 60;
var DAY = 1000 * 60 * 60 * 24;

// The maximum timer delay value is a 32-bit signed integer.
// See: https://mdn.io/setTimeout
var MAX_TIMER_DELAY = 2147483647;

function selectUnits(delta) {
    var absDelta = Math.abs(delta);

    if (absDelta < MINUTE) {
        return 'second';
    }

    if (absDelta < HOUR) {
        return 'minute';
    }

    if (absDelta < DAY) {
        return 'hour';
    }

    // The maximum scheduled delay will be measured in days since the maximum
    // timer delay is less than the number of milliseconds in 25 days.
    return 'day';
}

function getUnitDelay(units) {
    switch (units) {
        case 'second':
            return SECOND;
        case 'minute':
            return MINUTE;
        case 'hour':
            return HOUR;
        case 'day':
            return DAY;
        default:
            return MAX_TIMER_DELAY;
    }
}

var FormattedRelative = (function (_Component) {
    _inherits(FormattedRelative, _Component);

    function FormattedRelative(props, context) {
        _classCallCheck(this, FormattedRelative);

        _Component.call(this, props, context);
        _utils.invariantIntlContext(context);

        var now = isFinite(props.initialNow) ? Number(props.initialNow) : context.intl.now();

        // `now` is stored as state so that `render()` remains a function of
        // props + state, instead of accessing `Date.now()` inside `render()`.
        this.state = { now: now };
    }

    FormattedRelative.prototype.scheduleNextUpdate = function scheduleNextUpdate(props, state) {
        var _this = this;

        var updateInterval = props.updateInterval;

        // If the `updateInterval` is falsy, including `0`, then auto updates
        // have been turned off, so we bail and skip scheduling an update.
        if (!updateInterval) {
            return;
        }

        var delta = Number(props.value) - state.now;
        var units = props.units || selectUnits(delta);

        var unitDelay = getUnitDelay(units);
        var unitRemainder = Math.abs(delta % unitDelay);

        // We want the largest possible timer delay which will still display
        // accurate information while reducing unnecessary re-renders. The delay
        // should be until the next "interesting" moment, like a tick from
        // "1 minute ago" to "2 minutes ago" when the delta is 120,000ms.
        var delay = delta < 0 ? Math.max(updateInterval, unitDelay - unitRemainder) : Math.max(updateInterval, unitRemainder);

        clearTimeout(this._timer);

        this._timer = setTimeout(function () {
            _this.setState({ now: _this.context.intl.now() });
        }, delay);
    };

    FormattedRelative.prototype.shouldComponentUpdate = function shouldComponentUpdate() {
        for (var _len = arguments.length, next = Array(_len), _key = 0; _key < _len; _key++) {
            next[_key] = arguments[_key];
        }

        return _utils.shouldIntlComponentUpdate.apply(undefined, [this].concat(next));
    };

    FormattedRelative.prototype.componentWillUpdate = function componentWillUpdate(nextProps, nextState) {
        this.scheduleNextUpdate(nextProps, nextState);
    };

    FormattedRelative.prototype.componentDidMount = function componentDidMount() {
        this.scheduleNextUpdate(this.props, this.state);
    };

    FormattedRelative.prototype.componentWillUnmount = function componentWillUnmount() {
        clearTimeout(this._timer);
    };

    FormattedRelative.prototype.render = function render() {
        var formatRelative = this.context.intl.formatRelative;
        var _props = this.props;
        var value = _props.value;
        var children = _props.children;

        var formattedRelative = formatRelative(value, _extends({}, this.props, this.state));

        if (typeof children === 'function') {
            return children(formattedRelative);
        }

        return _react2['default'].createElement(
            'span',
            null,
            formattedRelative
        );
    };

    return FormattedRelative;
})(_react.Component);

exports['default'] = FormattedRelative;

FormattedRelative.displayName = 'FormattedRelative';

FormattedRelative.contextTypes = {
    intl: _types.intlShape
};

FormattedRelative.propTypes = _extends({}, _types.relativeFormatPropTypes, {
    value: _react.PropTypes.any.isRequired,
    format: _react.PropTypes.string,
    updateInterval: _react.PropTypes.number,
    initialNow: _react.PropTypes.any,
    children: _react.PropTypes.func
});

FormattedRelative.defaultProps = {
    updateInterval: 1000 * 10
};
module.exports = exports['default'];